#ifndef __CEPAPI_WINDOW_H__
#	define __CEPAPI_WINDOW_H__

#	include "x/api.h"
#	include "x/win32/win32.h"
#	include "x/type.h"

//#	include "cslib.h"

typedef unsigned short t_char;

struct s_window;

typedef struct s_window_ops {
	/* create and destroy
	 * return 0 for success
	*/
	int (*on_create)(struct s_window *win);
	int (*on_destroy)(struct s_window *win);

	/* initialize and finish */

	/* invoked by window_get_size, get attributes
	 * if width or height is zero, means can't deside now
	 * return 0 for success
	*/
	int (*on_get_size)(struct s_window *win, int *width, int *height);

	/* invoked by window_set_shape, set attributes
	 * return 0 if successfully process
	*/
	int (*on_set_shape)(struct s_window *win);

	/* mouse event
	 * return 0 if process the command
	*/
	int (*on_click)(struct s_window *win, int x, int y);

	/* in order to compatible with touch screen, does not use double click
	int (*mouse_dbl_click)(struct s_window *win, int keycode, int x, int y);
	*/

	/* drag may have two actions
	 *
	 *   1 drag son element moving in the region (response by father)
	 *   2 scroll display area                   (response by this node)
	 *
	 * drag_begin: return 0 for drag active, return -1 for drag not active
	 * drag_update: return 0 for updated correct, return -1 for error
	 * drag_end: return 0 for end corrently, return -1 for error
	*/
	int (*on_drag_begin)(struct s_window *win, int x, int y);//, struct s_window *tar_win);
	int (*on_drag_update)(struct s_window *win, int x, int y);
	int (*on_drag_end)(struct s_window *win, int x, int y);
	/* return 0 if process the command
	*/
	int (*on_move_over)(struct s_window *win, int x, int y);
	int (*on_move_in)(struct s_window *win);
	int (*on_move_out)(struct s_window *win);

	/* return 0 if process the scroll
	*/
	int (*on_scroll)(struct s_window *win, int dx, int dy);

	/* keyboard event
	 * return 0 if process the keyboard event
	*/
	int (*on_press)(struct s_window *win, int keycode);
	int (*on_release)(struct s_window *win, int keycode);
	int (*on_input)(struct s_window *win, t_char ascii_code);

	/* display callback
	 * return 0 if something has been updated
	*/
	int (*on_update)(struct s_window *win);

	/* command message
	 * return 0 if need to stop dispatch
	*/
	int (*on_command)(struct s_window *win, int command_code, void *param, int param_len);

	int (*on_active)(struct s_window *win);
	int (*on_disactive)(struct s_window *win);

	/* timer message */
	int (*on_timer)(struct s_window *win, int id);

	int (*on_idle)(struct s_window *win);

	int (*on_get_extra_size)(struct s_window *win, unsigned long *o_extra_bytes);

	int (*on_get_window_name)(struct s_window *win, const char **o_name);
}t_window_ops;

typedef struct s_window_data {
	/* link */
	struct s_window *father;
	struct s_window *son;
	struct s_window *son_previous;
	struct s_window *son_next;

	/* flags */
	int hide;

	/* window id */
	int id;

	/* window shape
	 * in father's coordinate
	*/
	int left, top;
	int right, bottom;

	/* original dot (x, y) */
	int orig_x, orig_y;

	/* content */
	int content_left, content_top, content_right, content_bottom;

	/* scroll initial (x, y) */
	int scroll_enable, scroll_x, scroll_y;

	/* system control handle */
	HMODULE hModule;

	HWND hMainWnd;
	HDC hMemoryDC;

	/* command parameter */
	struct s_window *source_win;
	void *param;
	int param_len;

	/* user data */
//	unsigned long user_data_size;
//	unsigned char user_data[4];
}t_window_data;

typedef struct s_window_info {
	const char *window_class_name;

	/* user data size in bytes */
	unsigned int extra_bytes;

	/* operations */
	t_window_ops ops;
}t_window_info;

typedef struct s_window {
//	t_cslib_api cslib_api;

	t_window_info info;

	const char *window_class_name;

	/* user data size in bytes */
	unsigned int extra_bytes;

	/* operations */
//	const t_window_ops *ops;
	t_window_ops ops;

	/* window data */
	t_window_data data;

	/* user extent data */
	uint8 *user_data;
	uint8 user_data4[4];
}t_window;

typedef struct s_rectangle {
	int left, top;
	int right, bottom;
}t_rectangle;

typedef struct s_size {
	int width, height;
}t_size;

typedef struct s_tag_size {
	int left_top_x;
	int left_top_y;
	int left_bottom_x;
	int left_bottom_y;
	int right_top_x;
	int right_top_y;
	int right_bottom_x;
	int right_bottom_y;

	int text_x;
	int text_y;

	int width;
	int height;

	int view_width;
}t_tag_size;

/* initialize global window data
*/
KAILIN_API int window_init(void);

/* finish global window
*/
KAILIN_API int window_exit(void);

KAILIN_API int window_create_new(t_window **o_win, const t_window_info *info);

KAILIN_API int window_destroy_new(t_window **io_win);

KAILIN_API int window_set_parameter(t_window *win, void *param, int param_len);

/* allocate space of window data and son's
*/
KAILIN_API int window_create(t_window *win, t_window *father, void *param, int param_len);

/* free space of window data and son's
*/
KAILIN_API int window_destroy(t_window *win);

KAILIN_API t_window *window_get_root(t_window *win);

/* get window's video buffer width and height
*/
KAILIN_API int window_get_width(t_window *win);
KAILIN_API int window_get_height(t_window *win);

KAILIN_API int window_get_id(t_window *win);
KAILIN_API int window_is_identify(t_window *win, int id);

/* get window's content size
*/
KAILIN_API int window_get_size(t_window *win, int *width, int *height);
KAILIN_API int window_get_parameter(t_window *win, void *o_param, int bytes);
KAILIN_API int window_get_user_data(t_window *win, void **o_data, unsigned int bytes);
KAILIN_API int window_get_extra_size(t_window *win, unsigned long *o_bytes);
KAILIN_API int window_get_window_name(t_window *win, const char **o_name);

KAILIN_API int window_set_main_wnd(t_window *win, HWND hWnd); /* only for root window */
KAILIN_API int window_set_father(t_window *win, t_window *father); /* only for derived window */
KAILIN_API int window_set_shape(t_window *win, int left, int top, int right, int bottom); /* only for derived window */
KAILIN_API int window_set_module(t_window *win, HMODULE hModule);
KAILIN_API int window_set_file(t_window *win, const char *filename);
KAILIN_API int window_set_size(t_window *win, int width, int height);
KAILIN_API int window_set_hide(t_window *win, int hide);

/* window set center normally used in on_update routine */
KAILIN_API int window_set_center(t_window *win);

/* dx, dy value
 *     = 0 to scroll null
 *     > 0 to scroll right or bottom
 *     < 0 to scroll left or top
 * as scroll accomplete, not to update screen
*/
KAILIN_API int window_scroll(t_window *win, int dx, int dy);

KAILIN_API int window_is_point_in(t_window *win, int x, int y);

/* window_do_??? used in it's other message except ??? message
 *************************************************************
*/

/* update display buffer from this to father and so on, finally update screen
 * used in other window message to update the change to screen
*/
KAILIN_API int window_do_update(t_window *win);
/* use window_do_command to dispatch command to all window object
 * from son to father
*/
KAILIN_API int window_do_command(t_window *win, int command_code, void *param, int param_len);
KAILIN_API int window_do_response(t_window *win, int command_code, void *param, int param_len);
/* close window
*/
KAILIN_API int window_do_close(t_window *win);

/* window_on_??? used in it's message ???
 ****************************************
*/

/* send command to son window
*/
KAILIN_API int window_on_command(t_window *win, int code, void *param, int param_len);
/* use window_do_paint to transfer root window buffer to DC
 * use in WM_PAINT message
*/
KAILIN_API int window_on_paint(t_window *win);
/* update this display buffer from this to sons and so on, and update father's buffer
 * used in update message
*/
KAILIN_API int window_on_update(t_window *win); /* update this buffer content */
//int window_paint(t_window *win); /* update this buffer to father buffer */
//int window_paint_to_father(t_window *win);
/* use window_on_click to dispatch mouse click from father to son
*/
KAILIN_API int window_on_click(t_window *win, int x, int y);
/* use window_on_drag_??? to dispatch mouse drag from father to son
*/
KAILIN_API int window_on_drag_begin(t_window *win, int x, int y);
KAILIN_API int window_on_drag_update(t_window *win, int x, int y);
KAILIN_API int window_on_drag_end(t_window *win, int x, int y);
/* use window_on_mouse to dispatch mouse over from father to son
*/
KAILIN_API int window_on_mouse_over(t_window *win, int x, int y);

KAILIN_API int window_on_press_key(t_window *win, int code);
KAILIN_API int window_on_release_key(t_window *win, int code);
KAILIN_API int window_on_input_char(t_window *win, t_char achar);

KAILIN_API int window_on_active(t_window *win);
KAILIN_API int window_on_disactive(t_window *win);

KAILIN_API int window_on_timer(t_window *win, int id);
KAILIN_API int window_on_idle(t_window *win);

/* draw graphics element
 * only valid after window_set_shape successfully
 * better used in on_update message
 ************************************************
*/

KAILIN_API int window_draw_empty(t_window *win, COLORREF color);
KAILIN_API int window_draw_char(t_window *win, int x, int y, int achar, COLORREF color);
KAILIN_API int window_draw_text(t_window *win, int x, int y, const char *text, int len, t_rectangle *rect);
KAILIN_API int window_draw_text_middle(t_window *win, int x, int y, const char *text);
KAILIN_API int window_draw_button(t_window *win, int x, int y, const char *text, int down, COLORREF color, COLORREF dkcolor, COLORREF pencolor, t_rectangle *rect);
KAILIN_API int window_draw_rectangle(t_window *win, int left, int top, int right, int bottom, COLORREF bkcolor, int bkshow, COLORREF lncolor, int lnwidth);
KAILIN_API int window_draw_triangle(t_window *win, int x1, int y1, int x2, int y2, int x3, int y3, COLORREF bkcolor, int bkshow, COLORREF lncolor, int lnwidth);
KAILIN_API int window_draw_line(t_window *win, int x1, int y1, int x2, int y2, COLORREF color, int width);
KAILIN_API int window_draw_tag(t_window *win, int x, int y, const char *title, int over, int open, COLORREF color, COLORREF dk_color, t_tag_size *psize);
KAILIN_API int window_draw_icon(t_window *win, int x, int y, HICON hIcon);

KAILIN_API int window_get_char_size(t_window *win, int achar, t_size *size);
KAILIN_API int window_get_text_size(t_window *win, const char *text, int len, t_size *size);
KAILIN_API int window_get_button_size(t_window *win, const char *text, t_size *size);
KAILIN_API int window_get_tag_size(t_window *win, const char *title, t_tag_size *psize);

/* debug function
 ****************
*/

KAILIN_API int window_show_error_box(t_window *win, const char *title, const char *text, ...);
KAILIN_API const char *window_name(t_window *win);

/* wrapped WIN32 function
 ****************************
*/

KAILIN_API int window_set_cursor(t_window *win, HCURSOR hCursor, HCURSOR *phOldCursor);
KAILIN_API int window_load_bitmap_file(const char *filename, HBITMAP *o_hBitmap);
KAILIN_API int window_load_bitmap(t_window *win, int rc_id, HBITMAP *o_hBitmap);
KAILIN_API int window_free_bitmap(HBITMAP *io_hBitmap);

KAILIN_API int window_load_icon_file(const char *filename, HICON *o_hIcon);
KAILIN_API int window_load_icon(t_window *win, int rc_id, HICON *o_hIcon);
KAILIN_API int window_load_first_icon(HMODULE hModule, HICON *o_hIcon);
KAILIN_API int window_free_icon(HICON *io_hIcon);

KAILIN_API int window_get_bitmap_size(HBITMAP hBitmap, t_size *size);
KAILIN_API int window_get_icon_size(HICON hIcon, t_size *size);
KAILIN_API int window_create_caret(t_window *win, int insert);
KAILIN_API int window_destroy_caret(t_window *win);
KAILIN_API int window_set_caret_pos(t_window *win, int x, int y);
KAILIN_API int window_show_caret(t_window *win);
KAILIN_API int window_hide_caret(t_window *win);

KAILIN_API int window_settimer(t_window *win, unsigned long ms);
KAILIN_API int window_killtimer(t_window *win);

#endif
