#ifndef __CSLIB_H__
#	define __CSLIB_H__

#	include "x/api.h"
#	include "x/null.h"
#	include "x/type.h"
#	include "x/dbg/debug.h"
#	include "x/decoder/mp3.h"
#	include "x/decoder/mp3_file.h"
#	include "x/decoder/mp3_play.h"
#	include "x/direct/direct.h"
#	include "x/memory/memory.h"
#	include "x/stream/stream.h"
#	include "x/win32/win32.h"

typedef struct s_cslib_api {
	/* stream.h */
	int (*stream_open)(STREAM **stream, const char *url);
	int (*stream_close)(STREAM **stream);
	int (*stream_read)(STREAM *stream, void *buffer, STREAM_BYTES required_bytes);
	int (*stream_get_length)(STREAM *stream, STREAM_BYTES *result_bytes);
	int (*stream_skip)(STREAM *stream, STREAM_BYTES required_bytes);
	int (*stream_get_name)(void *stream, char **result_name);

	int (*stream_read_init)(STREAM *stream, STREAM_BYTES buff_len);
	int (*stream_read_exit)(STREAM *stream);
	int (*stream_read_ahead)(STREAM *stream, void *buffer, STREAM_BYTES required_bytes);
	int (*stream_read_space)(STREAM *stream, STREAM_BYTES required_bytes);
	int (*stream_read_rollback)(STREAM *stream, STREAM_BYTES required_bytes);
	int (*stream_read_direct)(STREAM *stream, void *buffer, STREAM_BYTES required_bytes);
	int (*stream_read_rollback_all)(STREAM *stream);
	int (*stream_read_space_all)(STREAM *stream);
	int (*stream_read_clean)(STREAM *stream);

	/* memory.h */
	int (*mem_init)(void);
	int (*mem_exit)(void);

	int (*mem_alloc)(void **memblk, t_memory_bytes bytes, const char *msg);
	int (*mem_free)(void **memblk);

	int (*mem_get_size)(void *memblk, t_memory_bytes *o_bytes);
	int (*mem_test)(void *memblk);

	/* direct.h */
	int (*direct_open)(t_dir *dir, const char *path, const char *filter);
	int (*direct_close)(t_dir *dir);
	int (*direct_read_dirent)(t_dir *dir, t_dirent *dirent);
	int (*direct_get_path)(t_dir *dir, t_dirent *dirent, uint8 *buff, uint32 len);
	int (*direct_get_path_length)(t_dir *dir, t_dirent *dirent, uint32 *o_len);

	/* mp3_play.h */
	int (*mp3_open)(t_mp3_file *mp3_file, const char *file, t_mp3_play_cb *ops, void *ops_p);
	int (*mp3_close)(t_mp3_file *mp3_file);
	int (*mp3_pause)(t_mp3_file *mp3_file);
	int (*mp3_resume)(t_mp3_file *mp3_file);
	int (*mp3_play)(t_mp3_file *mp3_file);
	int (*mp3_stop)(t_mp3_file *mp3_file);
	int (*mp3_wait)(t_mp3_file *mp3_file);
	int (*mp3_playlist_init)(t_mp3_playlist *pl, t_mp3_play_cb *ops, void *ops_p);
	int (*mp3_playlist_insert)(t_mp3_playlist *pl, const char *filepath);
	int (*mp3_playlist_release)(t_mp3_playlist *pl);
	int (*mp3_playlist_play)(t_mp3_playlist *pl);
	int (*mp3_playlist_stop)(t_mp3_playlist *pl);
	int (*mp3_playlist_pause)(t_mp3_playlist *pl);
	int (*mp3_playlist_resume)(t_mp3_playlist *pl);
	int (*mp3_playlist_wait)(t_mp3_playlist *pl);
	int (*mp3_playlist_close)(t_mp3_playlist *pl);

	/* mp3.h */
	int (*mp3_get_id3v1)(STREAM *stream, ID3V1 *content);
	char *(*mp3_get_id3v1_content_type_name)(int id);
	int (*mp3_get_id3v2_header)(STREAM *stream, ID3V2_HEADER *header);
	int (*mp3_get_id3v2_frame_head)(STREAM *stream, ID3V2_FRAME *header);
	int (*mp3_get_id3v2_frame_body)(STREAM *stream, ID3V2_FRAME *header, void *body);
	long (*mp3_get_id3v2_frame_size)(ID3V2_HEADER *header);
	long (*mp3_get_id3v2_frame_body_size)(ID3V2_FRAME *frame);
	int (*mp3_get_id3v2_frame_id)(ID3V2_FRAME *header);
	char *(*mp3_get_id3v2_frame_desc)(int frame_id);
	char *(*mp3_get_id3v2_frame_content)(int frame_id, void *body);
	long (*mp3_get_length)(STREAM *stream);
	int (*mp3_get_audio_info)(/*MPEG *mpeg, */void *buff, unsigned int length, unsigned long *bytes, MP3_AUDIO_INFO *info);
	int (*mp3_get_audio_decode_buffer_length)(const MP3_AUDIO_INFO *info, unsigned long milli_seconds, unsigned long *bytes);
	int (*mp3_decode_one_audio_frame)(/*MPEG *mpeg, */const void *raw_data, unsigned long length, void *pcm, unsigned long *bytes);
	int (*mp3_decode_audio)(/*MPEG *mpeg, */unsigned long milli_seconds, const void *raw_data, unsigned long length, void *pcm, unsigned long *bytes);

	/* mp3_file.h */
	int (*mp3_file_read_id3v1)(STREAM *file, unsigned long offset, unsigned long file_size, t_mp3_id3v1 *o_id);
	int (*mp3_file_read_id3v2)(STREAM *file, unsigned long offset, t_mp3_id3v2 *tag);
	int (*mp3_file_free_id3v2)(t_mp3_id3v2 *tag);
	int (*mp3_file_read_frame)(STREAM *file, unsigned long offset, t_mp3_frame *o_frame);
	int (*mp3_file_read_frame_body)(STREAM *file, t_mp3_frame *frame, void *data);
	int (*mp3_file_get_frame_xing)(STREAM *file, t_mp3_frame *frame, const unsigned char *data, t_mp3_frame_xing *o_xing);

	/* debug.h */
	int (*debug_init)(const char *logfile);
	int (*debug_exit)(void);
	int (*debug_begin)(const char *fmt, ...);
	int (*debug_end)(const char *fmt, ...);
	int (*debug_print)(const char *fmt, ...);
	int (*debug_success)(void);
	int (*debug_failure)(void);
	int (*debug_finish)(void);
	int (*debug_on)(void);
	int (*debug_off)(void);
	int (*debug_pause)(void);

	/* win32.h */
	ATOM    (*Win32RegisterClassEx)(WNDCLASSEX *wc);
	HWND    (*Win32CreateWindow)(LPCTSTR lpClassName,
							   LPCTSTR lpWindowName, DWORD dwStyle,
							   int x, int y, int nWidth, int nHeight,
							   HWND hWndParent, HMENU hMenu, HINSTANCE hInstance,
							   LPVOID lpParam);
	BOOL    (*Win32ShowWindow)(HWND hWnd, int nShowCmd);
	BOOL    (*Win32UpdateWindow)(HWND hWnd);
	BOOL    (*Win32DestroyWindow)(HWND hWnd);
	int     (*Win32GetMessage)(MSG *msg, HWND hWnd, UINT min, UINT max);
	BOOL    (*Win32TranslateMessage)(LPMSG msg);
	LRESULT (*Win32DispatchMessage)(LPMSG msg);
	int     (*Win32PostQuitMessage)(int exitcode);
	LRESULT (*Win32DefWindowProc)(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
	int     (*Win32SetCapture)(HWND hWnd);
	int     (*Win32ReleaseCapture)(void);
	int     (*Win32GetClientRect)(HWND hWnd, LPRECT o_rect);
	HICON   (*Win32LoadIconRes)(HINSTANCE hInst, int nID);
	BOOL    (*Win32PeekMessage)(LPMSG lpMsg, HWND hWnd, UINT uMin, UINT uMax, UINT uRemoval);
	BOOL    (*Win32PostMessage)(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
	void    (*Win32Sleep)(DWORD dwMilliSeconds);
	HGDIOBJ (*Win32GetStockObject)(INT nIndex);
	HCURSOR (*Win32LoadCursor)(HINSTANCE hInstance, LPCTSTR lpCursorName);
	HICON   (*Win32LoadIcon)(HINSTANCE hInstance, LPCTSTR lpCursorName);
}t_cslib_api;

KAILIN_API int cslib_set_api(t_cslib_api *api);

#endif
