#ifndef __MP3_H__
#define __MP3_H__

#include "x/api.h"
#include "x/stream/stream.h"

/****************************************************************************/

enum {
#define DEF(n,id,eng) GENRE_##id=n,
#include "genre.h"
};

typedef struct tagID3V1 {
	char header[3];    /*ǩͷ"TAG"Ϊûбǩ*/
	char title[30];    /**/
	char artist[30];   /**/
	char album[30];    /*ר*/
	char year[4];      /*Ʒ*/
	char comment[28];  /*ע*/
	char reserve;      /**/
	char track;        /**/
	char genre;        /**/
}ID3V1;

KAILIN_API
 int mp3_get_id3v1(STREAM *stream, ID3V1 *content);
KAILIN_API
 char *mp3_get_id3v1_content_type_name(int id);

/****************************************************************************/

enum {
#define DEF(name,desc) FRAME_##name,
#include "frame.h"
};

#pragma pack(1)

typedef struct tagID3V2_HEADER {
	char header[3];  /*Ϊ"ID3"Ϊǩ*/
	char ver;        /*汾ID3V2.3 ͼ¼3*/
	char revision;   /*汾Ŵ˰汾¼Ϊ0*/
	char flag;       /*ű־ֽڣ汾ֻλԺϸ˵*/
	char size[4];    /*ǩСǩͷ10 ֽںеıǩ֡ĴС*/
}ID3V2_HEADER;

typedef struct tagID3V2_FRAME {
	char frame_id[4];  /*ĸַʶһ֡˵ݣԺгõıʶձ*/
	char size[4];      /*֡ݵĴС֡ͷС1*/
	char flags[2];     /*ű־ֻ6 λԺϸ˵*/
}ID3V2_FRAME;

#pragma pack()

KAILIN_API
 int mp3_get_id3v2_header(STREAM *stream, ID3V2_HEADER *header);
KAILIN_API
 int mp3_get_id3v2_frame_head(STREAM *stream, ID3V2_FRAME *header);
KAILIN_API
 int mp3_get_id3v2_frame_body(STREAM *stream, ID3V2_FRAME *header, void *body);
KAILIN_API
 long mp3_get_id3v2_frame_size(ID3V2_HEADER *header);
KAILIN_API
 long mp3_get_id3v2_frame_body_size(ID3V2_FRAME *frame);
KAILIN_API
 int mp3_get_id3v2_frame_id(ID3V2_FRAME *header);
KAILIN_API
 char *mp3_get_id3v2_frame_desc(int frame_id);
KAILIN_API
 char *mp3_get_id3v2_frame_content(int frame_id, void *body);

/****************************************************************************/

typedef struct MP3_AUDIO_INFO {
	/* MP3 media decode information */
	unsigned long  bitrate; /* mp3 bit rate, 0 for VBR */
	unsigned long  sample_rate;
	unsigned short sample_bits;
	unsigned short channels;

	/* buffer length needed for decoded PCM data */
	unsigned long  decode_buffer_length; /* bytes per frame */

	/* CBR and VBR shared fields */
	unsigned long  file_bytes;
	unsigned long  frames;

	/* CBR used only fields */
	/* when frame exist pad bit, then pading bytes should be added */
	unsigned long  cbr_frame_length; /* bytes per frame */

	/* VBR used only fields */
	unsigned long  vbr_flags;
#define MP3_AUDIO_FLAGS_VBR_FRAMES      1 /* audio frames */
#define MP3_AUDIO_FLAGS_VBR_BYTES       2 /* file length */
#define MP3_AUDIO_FLAGS_VBR_TOC         4 /* has VBR TOC */
#define MP3_AUDIO_FLAGS_VBR_SCALE       8 /* audio quality */
	unsigned long  vbr_scale;
	unsigned char  vbr_toc[100];
}MP3_AUDIO_INFO;

KAILIN_API
 long mp3_get_length(STREAM *stream);
KAILIN_API
 int mp3_get_audio_info(/*MPEG *mpeg, */void *buff, unsigned int length, unsigned long *bytes, MP3_AUDIO_INFO *info);
KAILIN_API
 int mp3_get_audio_decode_buffer_length(const MP3_AUDIO_INFO *info, unsigned long milli_seconds, unsigned long *bytes);
KAILIN_API
 int mp3_decode_one_audio_frame(/*MPEG *mpeg, */const void *raw_data, unsigned long length, void *pcm, unsigned long *bytes);
KAILIN_API
 int mp3_decode_audio(/*MPEG *mpeg, */unsigned long milli_seconds, const void *raw_data, unsigned long length, void *pcm, unsigned long *bytes);


//typedef struct s_mp3_ops {
//	int (*mp3_get_id3v1)(STREAM *stream, ID3V1 *content);
//	char *(*mp3_get_id3v1_content_type_name)(int id);

//	int (*mp3_get_id3v2_header)(STREAM *stream, ID3V2_HEADER *header);
//	int (*mp3_get_id3v2_frame_head)(STREAM *stream, ID3V2_FRAME *header);
//	int (*mp3_get_id3v2_frame_body)(STREAM *stream, ID3V2_FRAME *header, void *body);
//	long (*mp3_get_id3v2_frame_size)(ID3V2_HEADER *header);
//	long (*mp3_get_id3v2_frame_body_size)(ID3V2_FRAME *frame);
//	int (*mp3_get_id3v2_frame_id)(ID3V2_FRAME *header);
//	char *(*mp3_get_id3v2_frame_desc)(int frame_id);
//	char *(*mp3_get_id3v2_frame_content)(int frame_id, void *body);

//	long (*mp3_get_length)(STREAM *stream);
//	int (*mp3_get_audio_info)(/*MPEG *mpeg, */void *buff, unsigned int length, unsigned long *bytes, MP3_AUDIO_INFO *info);
//	int (*mp3_get_audio_decode_buffer_length)(const MP3_AUDIO_INFO *info, unsigned long milli_seconds, unsigned long *bytes);
//	int (*mp3_decode_one_audio_frame)(/*MPEG *mpeg, */const void *raw_data, unsigned long length, void *pcm, unsigned long *bytes);
//	int (*mp3_decode_audio)(/*MPEG *mpeg, */unsigned long milli_seconds, const void *raw_data, unsigned long length, void *pcm, unsigned long *bytes);

//}t_mp3_ops;

//int mp3_get_ops(t_mp3_ops *o_ops);


#endif
