#ifndef __MP3_FILE_H__
#define __MP3_FILE_H__

#include "x/api.h"
#include "x/stream/stream.h"

enum { /* define ID3V1.genre */
#define DEF(i,n,s) MP3_ID3V1_GENRE_##n=i,
#include "genre.h"
};

typedef struct s_mp3_id3v1 {
	unsigned long offset;
	unsigned long size;

	unsigned char title[31];
	unsigned char artist[31];
	unsigned char album[31];
	unsigned char year[5];
	unsigned char comment[30];
	unsigned char track;
	unsigned char genre;
	const char *genre_text;
}t_mp3_id3v1;

#define MP3_ID3V1_OFFSET         -128
#define MP3_ID3V1_LENGTH                128
#define MP3_ID3V1_TAG_TEXT                        "TAG"
#define MP3_ID3V1_TAG_OFFSET        0
#define MP3_ID3V1_TAG_LENGTH              3
#define MP3_ID3V1_TITLE_OFFSET      3
#define MP3_ID3V1_TITLE_LENGTH           30
#define MP3_ID3V1_ARTIST_OFFSET    33
#define MP3_ID3V1_ARTIST_LENGTH          30
#define MP3_ID3V1_ALBUM_OFFSET     63
#define MP3_ID3V1_ALBUM_LENGTH           30
#define MP3_ID3V1_YEAR_OFFSET      93
#define MP3_ID3V1_YEAR_LENGTH             4
#define MP3_ID3V1_COMMENT_OFFSET   97
#define MP3_ID3V1_COMMENT_LENGTH         29
#define MP3_ID3V1_TRACK_OFFSET    126
#define MP3_ID3V1_TRACK_LENGTH            1
#define MP3_ID3V1_GENRE_OFFSET    127
#define MP3_ID3V1_GENRE_LENGTH            1


enum { /* define ID3V2.frame.id */
#define DEF(n,s) MP3_ID3V2_FRAME_##n,
#include "frame.h"
};

typedef struct s_mp3_id3v2_frame {
	struct s_mp3_id3v2_frame *previous, *next;
	unsigned long offset;
	unsigned long size;

	unsigned long id;
	const char *id_text;
	unsigned short flag;
#define MP3_ID3V2_FRAME_FLAG_TAG_PROTECTED      0x8000
#define MP3_ID3V2_FRAME_FLAG_FILE_PROTECTED     0x4000
#define MP3_ID3V2_FRAME_FLAG_READONLY           0x2000
#define MP3_ID3V2_FRAME_FLAG_COMPRESS           0x0080
#define MP3_ID3V2_FRAME_FLAG_ENCRYPTED          0x0040
#define MP3_ID3V2_FRAME_FLAG_GROUP              0x0020
	unsigned long data_size;
	unsigned char data[1];
}t_mp3_id3v2_frame;

#define MP3_ID3V2_FRAME_HEADER_OFFSET         0
#define MP3_ID3V2_FRAME_HEADER_LENGTH               10
#define MP3_ID3V2_FRAME_HEADER_ID_OFFSET      0
#define MP3_ID3V2_FRAME_HEADER_ID_LENGTH             4
#define MP3_ID3V2_FRAME_HEADER_SIZE_OFFSET    4
#define MP3_ID3V2_FRAME_HEADER_SIZE_LENGTH           4
#define MP3_ID3V2_FRAME_HEADER_FLAG_OFFSET    8
#define MP3_ID3V2_FRAME_HEADER_FLAG_LENGTH           2
#define MP3_ID3V2_FRAME_HEADER_FLAG_MASK                    0xE0E0

typedef struct s_mp3_id3v2 {
	unsigned long offset;
	unsigned long size;

	unsigned char version;
	unsigned char revision;
	unsigned char flag;
#define MP3_ID3V2_HEADER_FLAG_UNSYNCHRONISATION   0x80
#define MP3_ID3V2_HEADER_FLAG_HAS_EXTEND_HEADER   0x40
#define MP3_ID3V2_HEADER_FLAG_IS_TEST_TAG         0x20
	unsigned long frame_offset;
	unsigned long frame_size;
	t_mp3_id3v2_frame *frame_table;
}t_mp3_id3v2;

#define MP3_ID3V2_HEADER_OFFSET            0
#define MP3_ID3V2_HEADER_LENGTH                 10
#define MP3_ID3V2_HEADER_TAG_TEXT                      "ID3"
#define MP3_ID3V2_HEADER_TAG_OFFSET        0
#define MP3_ID3V2_HEADER_TAG_LENGTH              3
#define MP3_ID3V2_HEADER_VERSION_OFFSET    3
#define MP3_ID3V2_HEADER_VERSION_LENGTH          1
#define MP3_ID3V2_HEADER_REVISION_OFFSET   4
#define MP3_ID3V2_HEADER_REVISION_LENGTH         1
#define MP3_ID3V2_HEADER_FLAG_OFFSET       5
#define MP3_ID3V2_HEADER_FLAG_LENGTH             1
#define MP3_ID3V2_HEADER_FLAG_MASK                      0xE0
#define MP3_ID3V2_HEADER_SIZE_OFFSET       6
#define MP3_ID3V2_HEADER_SIZE_LENGTH             4
#define MP3_ID3V2_HEADER_SIZE_MASK                      0x7F
#define MP3_ID3V2_HEADER_SIZE_BASE_1                    0x200000UL
#define MP3_ID3V2_HEADER_SIZE_BASE_2                    0x4000UL
#define MP3_ID3V2_HEADER_SIZE_BASE_3                    0x80UL
#define MP3_ID3V2_HEADER_SIZE_BASE_4                    0x1UL

typedef struct s_mp3_frame {
	unsigned long offset;
	unsigned long size;

	unsigned long flags;
#define MP3_FRAME_FLAGS_MASK            0x000103C0UL
#define MP3_FRAME_FLAGS_CRC             0x00010000UL
#define MP3_FRAME_FLAGS_FILL_BIT        0x00000200UL
#define MP3_FRAME_FLAGS_PRIVATE         0x00000100UL
#define MP3_FRAME_FLAGS_COPYRIGHT       0x00000008UL
#define MP3_FRAME_FLAGS_SOURCE          0x00000004UL
	unsigned long bit_rate;
	unsigned long sample_rate;
	unsigned long samples;
	unsigned long frame_offset;
	unsigned long frame_size;
	unsigned long frame_us; /* micro seconds */
	unsigned char mpeg_version_major;
	unsigned char mpeg_version_minor;
	unsigned char layer;
	unsigned char channels;
	unsigned char stereo;
#define MP3_FRAME_CHANNEL_STEREO               0
#define MP3_FRAME_CHANNEL_JOINT_STEREO         1 /* stereo */
#define MP3_FRAME_CHANNEL_DUAL_CHANNEL         2 /* stereo */
#define MP3_FRAME_CHANNEL_MONO_CHANNEL         3
#define MP3_FRAME_CHANNEL_JOINT_STEREO_BAND_4                 5     /* layer I & II */
#define MP3_FRAME_CHANNEL_JOINT_STEREO_BAND_8                 6     /* layer I & II */
#define MP3_FRAME_CHANNEL_JOINT_STEREO_BAND_12                7     /* layer I & II */
#define MP3_FRAME_CHANNEL_JOINT_STEREO_BAND_16                8     /* layer I & II */
#define MP3_FRAME_CHANNEL_JOINT_STEREO_INTENSITY_STEREO             9  /* layer III */
#define MP3_FRAME_CHANNEL_JOINT_STEREO_MS_STEREO                   10  /* layer III */
#define MP3_FRAME_CHANNEL_JOINT_STEREO_MS_INTENSITY_STEREO         11  /* layer III */
	unsigned char  bass;
#define MP3_FRAME_BASS_NULL             0
#define MP3_FRAME_BASS_50_15_MS         1
#define MP3_FRAME_BASS_CCIT_J_17        3
}t_mp3_frame;

typedef struct s_mp3_frame_xing {
	unsigned long flags;
#define MP3_FRAME_XING_FLAGS_MASK                     0x0000000FUL
#define MP3_FRAME_XING_FLAGS_HAS_FRAMES               0x00000001UL
#define MP3_FRAME_XING_FLAGS_HAS_FILE_SIZE            0x00000002UL
#define MP3_FRAME_XING_FLAGS_HAS_TOC                  0x00000004UL
#define MP3_FRAME_XING_FLAGS_HAS_QULITY_INDICATOR     0x00000008UL
	unsigned long frames;
	unsigned long file_size;
	unsigned char toc[100];
	unsigned char qulity;
}t_mp3_frame_xing;

KAILIN_API
 int mp3_file_read_id3v1(STREAM *file, unsigned long offset, unsigned long file_size, t_mp3_id3v1 *o_id);
KAILIN_API
 int mp3_file_read_id3v2(STREAM *file, unsigned long offset, t_mp3_id3v2 *tag);
KAILIN_API
 int mp3_file_free_id3v2(t_mp3_id3v2 *tag);
KAILIN_API
 int mp3_file_read_frame(STREAM *file, unsigned long offset, t_mp3_frame *o_frame);
KAILIN_API
 int mp3_file_read_frame_body(STREAM *file, t_mp3_frame *frame, void *data);
KAILIN_API
 int mp3_file_get_frame_xing(STREAM *file, t_mp3_frame *frame, const unsigned char *data, t_mp3_frame_xing *o_xing);

//typedef struct s_mp3_file_ops {
//	int (*mp3_file_read_id3v1)(STREAM *file, unsigned long offset, unsigned long file_size, t_mp3_id3v1 *o_id);
//	int (*mp3_file_read_id3v2)(STREAM *file, unsigned long offset, t_mp3_id3v2 *tag);
//	int (*mp3_file_free_id3v2)(t_mp3_id3v2 *tag);
//	int (*mp3_file_read_frame)(STREAM *file, unsigned long offset, t_mp3_frame *o_frame);
//	int (*mp3_file_read_frame_body)(STREAM *file, t_mp3_frame *frame, void *data);
//	int (*mp3_file_get_frame_xing)(STREAM *file, t_mp3_frame *frame, const unsigned char *data, t_mp3_frame_xing *o_xing);
//}t_mp3_file_ops;

//int mp3_file_get_ops(t_mp3_file_ops *o_ops);

#endif
