#ifndef __MP3_PLAY_H__
#define __MP3_PLAY_H__

#include "x/api.h"
#include "x/win32/win32.h"
#include "x/stream/stream.h"

//#include <stdio.h>
//#include <windows.h>
//#include <mmsystem.h>

typedef struct s_mp3_play_info {
	char *artist;
	char *title;
	int hour;
	int minute;
	int second;
}t_mp3_play_info;

typedef struct s_mp3_play_cb {
	int (*on_open)(void *p);
	int (*on_close)(void *p);
	int (*on_play)(void *p, const char *artist, const char *title, int hour, int minute, int second);
}t_mp3_play_cb;

typedef struct s_mp3_file {
	WAVEHDR mm_WaveHdr[2];

	HWAVEOUT mm_hWaveOut;
	HANDLE hEvent;

//	t_mp3_play_info info;
	char *info_artist;
	char *info_title;

	unsigned long mm_buffer_length;
	short *mm_buffer[2];

	STREAM *stream;
	FILE *wave_fp;
	char *music; /* buffer to load all the encoded data */

	volatile unsigned long index;
	volatile unsigned long length;

	volatile int count;
	int second;
	int mm_index;

	volatile int running;

	t_mp3_play_cb *ops;
	void *ops_p;
}t_mp3_file;

KAILIN_API
 int mp3_open(t_mp3_file *mp3_file, const char *file, t_mp3_play_cb *ops, void *ops_p);
KAILIN_API
 int mp3_close(t_mp3_file *mp3_file);
KAILIN_API
 int mp3_pause(t_mp3_file *mp3_file);
KAILIN_API
 int mp3_resume(t_mp3_file *mp3_file);
KAILIN_API
 int mp3_play(t_mp3_file *mp3_file);
KAILIN_API
 int mp3_stop(t_mp3_file *mp3_file);
KAILIN_API
 int mp3_wait(t_mp3_file *mp3_file);

typedef struct s_mp3_playlist_node {
	struct s_mp3_playlist_node *previous, *next;

	int filepath_len;
	char filepath[4];
}t_mp3_playlist_node;

typedef struct s_mp3_playlist {
	t_mp3_playlist_node *playlist;
	t_mp3_playlist_node *current;
	t_mp3_file file;
	volatile int stopped;
	HANDLE hThread;
	t_mp3_play_cb *ops;
	void *ops_p;
}t_mp3_playlist;

KAILIN_API
 int mp3_playlist_init(t_mp3_playlist *pl, t_mp3_play_cb *ops, void *ops_p);
KAILIN_API
 int mp3_playlist_insert(t_mp3_playlist *pl, const char *filepath);
KAILIN_API
 int mp3_playlist_release(t_mp3_playlist *pl);
KAILIN_API
 int mp3_playlist_play(t_mp3_playlist *pl);
KAILIN_API
 int mp3_playlist_stop(t_mp3_playlist *pl);
KAILIN_API
 int mp3_playlist_pause(t_mp3_playlist *pl);
KAILIN_API
 int mp3_playlist_resume(t_mp3_playlist *pl);
KAILIN_API
 int mp3_playlist_wait(t_mp3_playlist *pl);
KAILIN_API
 int mp3_playlist_close(t_mp3_playlist *pl);

//typedef struct s_mp3_play_ops {
//	int (*mp3_open)(t_mp3_file *mp3_file, const char *file, t_mp3_play_cb *ops, void *ops_p);
//	int (*mp3_close)(t_mp3_file *mp3_file);
//	int (*mp3_pause)(t_mp3_file *mp3_file);
//	int (*mp3_resume)(t_mp3_file *mp3_file);
//	int (*mp3_play)(t_mp3_file *mp3_file);
//	int (*mp3_stop)(t_mp3_file *mp3_file);
//	int (*mp3_wait)(t_mp3_file *mp3_file);

//	int (*mp3_playlist_init)(t_mp3_playlist *pl, t_mp3_play_cb *ops, void *ops_p);
//	int (*mp3_playlist_insert)(t_mp3_playlist *pl, const char *filepath);
//	int (*mp3_playlist_release)(t_mp3_playlist *pl);
//	int (*mp3_playlist_play)(t_mp3_playlist *pl);
//	int (*mp3_playlist_stop)(t_mp3_playlist *pl);
//	int (*mp3_playlist_pause)(t_mp3_playlist *pl);
//	int (*mp3_playlist_resume)(t_mp3_playlist *pl);
//	int (*mp3_playlist_wait)(t_mp3_playlist *pl);
//	int (*mp3_playlist_close)(t_mp3_playlist *pl);
//}t_mp3_play_ops;

//int mp3_play_get_ops(t_mp3_play_ops *o_ops);

#endif
