/* field is used by network message protocol
 * it's behaviour is like HTTP protocol
 * but it is in binary
*/

/* field buffer map
 *
 * 0                                                                    n
 * +--------+-----------+---------+--------+-----------+--------+------+
 * | HEADER | PARAMETER | ... ... | HEADER | PARAMETER | HEADER | DATA |
 * +--------+-----------+---------+--------+-----------+--------+------+
 * | FIELD PARAMETER                                   | FIELD BODY    |
 *
 * field type must be complicity
*/

#if 0

/* field context
*/
typedef struct s_field {
	unsigned short len; /* length of field */
	unsigned char type; /* type of field */

	const char *pbuf; /* buffer pointer of field */

	/* INNER attributes */
	unsigned short index; /* buffer index of field */
}t_field;

/* get spcified field of the buffer
 *
 * field->type  [IN]
 * field->len       [OUT] set the length of data
 * field->pbuf      [OUT] point to the data of field
 * field->index     [OUT] set to the head of next field
*/
int f_field_get(const char *buf /*I*/, unsigned short n /*I*/, t_field *field /*IO*/);

/* field->type      [OUT]
 * field->len       [OUT]
 * field->pbuf      [OUT]
 * field->index [IN][OUT] must be set at initial call
*/
int f_field_read(const char *buf /*I*/, unsigned short n /*I*/, t_field *field /*IO*/);

/* field->type  [IN]
 * field->len   [IN]
 * field->pbuf  [IN]
 * field->index [IN][OUT] must be set at initial call
*/
int f_field_write(char *buf /*O*/, unsigned short n /*I*/, t_field *field /*IO*/);

/* field must be a valid field after get or read operation
*/
int f_field_delete(char *buf /*O*/, unsigned short n /*I*/, const t_field *field /*I*/);

typedef struct SFieldContext {
	TInt type; /* type of field */
	TInt id; /* id of field */

	TInt len; /* length of field */
	const void *pbuf; /* buffer pointer of field */

	/* INNER attributes */
	TInt index; /* buffer index of field */
}TFieldContext;

TInt FieldDelete(void *buf/*O*/, TInt len/*I*/, const TFieldContext *ctx/*I*/);
TInt FieldGetParameter(const void *buf/*I*/, TInt len/*I*/, TFieldContext *ctx/*IO*/);
TInt FieldReadParameter(const void *buf/*I*/, TInt len/*I*/, TFieldContext *ctx/*IO*/);
TInt FieldWriteParameter(void *buf/*O*/, TInt len/*I*/, TFieldContext *ctx/*IO*/);
TInt FieldReadBody(const void *buf/*I*/, TInt len/*I*/, TFieldContext *ctx/*IO*/);
TInt FieldWriteBody(void *buf/*O*/, TInt len/*I*/, TFieldContext *ctx/*IO*/);
TInt FieldCloseBody(void *buf/*O*/, TInt len/*I*/, TFieldContext *ctx);
TInt FieldGetBodyCount(const void *buf/*I*/, TInt len/*I*/, TInt *ocount/*O*/);
TInt FieldGetBodyLength(const void *buf/*I*/, TInt len/*I*/, TInt *olen/*O*/);
TInt FieldReadWholeBody(const void *buf/*I*/, TInt len/*I*/, void *obuf/*O*/);
TInt FieldDeleteWholeBody(void *buf/*O*/, TInt len/*I*/);

#endif

#if 0

typedef struct SField {
	TUInt id;
	TUInt length;
	const TInt8 *buff;
	TUInt offset;
}TFieldContext;

/* get next field of current field
 * fld->id          [OUT]
 * fld->length      [OUT]
 * fld->buff        [OUT]
 * fld->offset  [IN][OUT]  offset point to next field
*/
TInt GetNextField(const TInt8 *buff/*I*/, TUInt len/*I*/, TFieldContext *fld/*IO*/);

/* get specified field
 * fld->id      [IN]       can't be 0
 * fld->length      [OUT]
 * fld->buff        [OUT]
 * fld->offset  [IN][OUT]  offset point to next field
*/
TInt GetField(const TInt8 *buff/*I*/, TUInt len/*I*/, TFieldContext *fld/*IO*/);

/* create new field
 * fld->id      [IN]       can't be 0
 * fld->length  [IN]
 * fld->buff    [IN]
 * fld->offset  [IN][OUT]  offset point next to the last field
*/
TInt SetField(TInt8 *buff/*O*/, TUInt len/*I*/, TFieldContext *fld/*IO*/);

/* create new expendable field
 * fld->id      [IN]       can't be 0
 * fld->length  [IN]
 * fld->buff    [IN]
 * fld->offset  [IN][OUT]  offset point to next field
*/
TInt OpenField(TInt8 *buff/*O*/, TUInt len/*I*/, TFieldContext *fld/*IO*/);

/* close expendable field
 * fld->id          [OUT]
 * fld->length      [OUT]
 * fld->buff        [OUT]
 * fld->offset  [IN][OUT]  offset point next to the last field
*/
TInt CloseField(TInt8 *buff/*IO*/, TUInt len/*I*/, TFieldContext *fld/*IO*/);

/* Append data to expendable field
 * fld->id                 no use
 * fld->length  [IN]
 * fld->buff    [IN]
 * fld->offset  [IN][OUT]  offset point to next field
*/
TInt AppendFieldData(TInt *buff/*O*/, TUInt len/*I*/, TFieldContext *fld/*IO*/);

#endif

#include "x/api.h"
#include "integer.h"

typedef struct SFieldContext {
	Int8 *buffer;
	UInt length;
	UInt offset;
}TFieldContext;

typedef struct SField {
	Int id;
	UInt length;
	const UInt8 *buffer;
	UInt offset;
}TField;

KAILIN_API Int OpenFieldContext(TFieldContext *ctx, Int8 *buff, UInt length);
KAILIN_API Int CloseFieldContext(TFieldContext *ctx);
KAILIN_API Int GetNextField(TFieldContext *ctx, TField *field);
KAILIN_API Int GetField(TFieldContext *ctx, TField *field, Int id);
KAILIN_API Int SetFieldData(TFieldContext *ctx, TField *field, UInt offset, const void *data, UInt data_length);
KAILIN_API Int DeleteField(TFieldContext *ctx, TField *field);
KAILIN_API Int CreateField(TFieldContext *ctx, Int id, const void *buff, UInt length);
KAILIN_API Int CreateExpendableField(TFieldContext *ctx, TField *field, Int id);
KAILIN_API Int FinishExpendableField(TFieldContext *ctx, TField *field);
KAILIN_API Int ExpendFieldData(TFieldContext *ctx, TField *field, const void *data, UInt data_length);
