/* input only bytes stream, readonly, unseekable
 *
 * supported url grammer
 *
 *     <space><proto><space>:<space>//<space>
 *
 * supported url proto
 *
 *     1 file
 *     2 resource
 */

#ifndef __STREAM_H__
#	define __STREAM_H__

#include "x/api.h"

/* stream base type */

typedef unsigned long STREAM_BYTES; /* length */

/* stream functions */

typedef struct STREAM_FUNCTIONS {

	int (*stream_open)(void *stream, const char *url);
	int (*stream_close)(void *stream);
	int (*stream_read)(void *stream, void *buffer, STREAM_BYTES required_bytes);
	int (*stream_get_length)(void *stream, STREAM_BYTES *result_bytes);
	int (*stream_skip)(void *stream, STREAM_BYTES required_bytes);
	int (*stream_get_name)(void *stream, char **result_name);

}STREAM_FUNCTIONS;

/* stream member data */

typedef struct STREAM {

	STREAM_FUNCTIONS *functions;

	/* precached buffer mechanism */
	unsigned char *buffer;
	STREAM_BYTES buffer_length;
	STREAM_BYTES buffer_free_index;
	STREAM_BYTES buffer_free_len;
	STREAM_BYTES buffer_read_index;
}STREAM;

/* stream api */

KAILIN_API
 int stream_open(STREAM **stream, const char *url);
KAILIN_API
 int stream_close(STREAM **stream);
KAILIN_API
 int stream_read(STREAM *stream, void *buffer, STREAM_BYTES required_bytes);
KAILIN_API
 int stream_get_length(STREAM *stream, STREAM_BYTES *result_bytes);
KAILIN_API
 int stream_skip(STREAM *stream, STREAM_BYTES required_bytes);
KAILIN_API
 int stream_get_name(void *stream, char **result_name);

/* init precached buffer mechanism */
KAILIN_API
 int stream_read_init(STREAM *stream, STREAM_BYTES buff_len);
/* finish precached buffer mechanism */
KAILIN_API
 int stream_read_exit(STREAM *stream);
/* read from buffer or source */
KAILIN_API
 int stream_read_ahead(STREAM *stream, void *buffer, STREAM_BYTES required_bytes);
/* accept the last readed words and move reading pointer to next */
KAILIN_API
 int stream_read_space(STREAM *stream, STREAM_BYTES required_bytes);
/* move reading pointer to last readed words */
KAILIN_API
 int stream_read_rollback(STREAM *stream, STREAM_BYTES required_bytes);
/* clear current cached buffer and read from the source */
KAILIN_API
 int stream_read_direct(STREAM *stream, void *buffer, STREAM_BYTES required_bytes);
KAILIN_API
 int stream_read_rollback_all(STREAM *stream);
KAILIN_API
 int stream_read_space_all(STREAM *stream);
KAILIN_API
 int stream_read_clean(STREAM *stream);

//typedef struct s_stream_ops {
/* stream api */

//	int (*stream_open)(STREAM **stream, const char *url);
//	int (*stream_close)(STREAM **stream);
//	int (*stream_read)(STREAM *stream, void *buffer, STREAM_BYTES required_bytes);
//	int (*stream_get_length)(STREAM *stream, STREAM_BYTES *result_bytes);
//	int (*stream_skip)(STREAM *stream, STREAM_BYTES required_bytes);
//	int (*stream_get_name)(void *stream, char **result_name);

/* init precached buffer mechanism */
//	int (*stream_read_init)(STREAM *stream, STREAM_BYTES buff_len);
/* finish precached buffer mechanism */
//	int (*stream_read_exit)(STREAM *stream);
/* read from buffer or source */
//	int (*stream_read_ahead)(STREAM *stream, void *buffer, STREAM_BYTES required_bytes);
/* accept the last readed words and move reading pointer to next */
//	int (*stream_read_space)(STREAM *stream, STREAM_BYTES required_bytes);
/* move reading pointer to last readed words */
//	int (*stream_read_rollback)(STREAM *stream, STREAM_BYTES required_bytes);
/* clear current cached buffer and read from the source */
//	int (*stream_read_direct)(STREAM *stream, void *buffer, STREAM_BYTES required_bytes);
//	int (*stream_read_rollback_all)(STREAM *stream);
//	int (*stream_read_space_all)(STREAM *stream);
//	int (*stream_read_clean)(STREAM *stream);
//}t_stream_ops;

//int stream_get_ops(t_stream_ops *o_ops);

#endif
