/* region file */

#ifndef __RFILE_H__
#define __RFILE_H__

#include "x/api.h"
#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef unsigned long RFILE_REGION_ID;
typedef unsigned long RFILE_REGION_SIZE;

struct rfile_region_header {
	RFILE_REGION_ID id; // id 0 is invalid id
//	unsigned short flags;
	RFILE_REGION_SIZE size;
};

struct rfile_map_node {
	struct rfile_map_node *previous, *next;
	struct rfile_region_header header;
	RFILE_REGION_SIZE offset;
};

typedef struct RFILE {
	FILE *file_pointer;
	struct rfile_map_node *map;
}RFILE;

KAILIN_API
 int rfile_open(RFILE *file, const char *file_name);
KAILIN_API
 int rfile_create(RFILE *file, const char *file_name);
KAILIN_API
 int rfile_close(RFILE *file);

KAILIN_API
 int rfile_region_create(RFILE *file, RFILE_REGION_ID id);
KAILIN_API
 int rfile_region_delete(RFILE *file, RFILE_REGION_ID id);
KAILIN_API
 int rfile_region_can_be_appended(RFILE *file, RFILE_REGION_ID id);
KAILIN_API
 int rfile_region_get_count(RFILE *file, RFILE_REGION_ID id, unsigned long *out_count);

KAILIN_API
 int rfile_overlay_read(RFILE *file, RFILE_REGION_ID id, RFILE_REGION_SIZE offset, void *buffer, unsigned long in_length, unsigned long *out_length);
KAILIN_API
 int rfile_overlay_write(RFILE *file, RFILE_REGION_ID id, RFILE_REGION_SIZE offset, const void *buffer, unsigned long in_length, unsigned long *out_length);
KAILIN_API
 int rfile_overlay_get_length(RFILE *file, RFILE_REGION_ID id, RFILE_REGION_SIZE *out_length);

KAILIN_API
 int rfile_serial_read(RFILE *file, RFILE_REGION_ID id, RFILE_REGION_SIZE position, void *buffer, unsigned long in_length, unsigned long *out_length);
KAILIN_API
 int rfile_serial_write(RFILE *file, RFILE_REGION_ID id, RFILE_REGION_SIZE position, void *buffer, unsigned long in_length, unsigned long *out_length);
KAILIN_API
 int rfile_serial_get_length(RFILE *file, RFILE_REGION_ID id, RFILE_REGION_SIZE *out_length);

KAILIN_API
 int rfile_seperate_read(RFILE *file, RFILE_REGION_ID id, unsigned long index, RFILE_REGION_SIZE position, void *buffer, unsigned long in_length, unsigned long *out_length);
KAILIN_API
 int rfile_seperate_write(RFILE *file, RFILE_REGION_ID id, unsigned long index, RFILE_REGION_SIZE position, const void *buffer, unsigned long in_length, unsigned long *out_length);
KAILIN_API
 int rfile_seperate_get_length(RFILE *file, RFILE_REGION_ID id, unsigned long index, RFILE_REGION_SIZE *out_length);

#ifdef __cplusplus
};
#endif

#endif
