#ifndef __WINGRAPH_H__
#define __WINGRAPH_H__

#include "x/api.h"
#include "x/null.h"
#include "x/type.h"

struct _RPC_ASYNC_STATE; /* supress warning message */
#include <windows.h>

#ifdef __cplusplus
extern "C" {
#endif

////////////////////////////////////////////////////////////////////////////

#define DEFAULT_NAME_LENGTH 42

struct s_type;
struct s_variable;
struct s_variable_bare;
struct s_const;

union u_base_type_data {
	signed int int_data;
	unsigned uint_data;
	signed char char_data;
	unsigned char uchar_data;
	signed short short_data;
	unsigned short ushort_data;
	signed long long_data;
	unsigned long ulong_data;
	float float_data;
	double double_data;
	long double ldouble_data;
	void *address_data;
	void (*function_data)(void);
};

union u_struct_type_data {
	struct s_type_struct {
		struct s_variable *vars; /* QUEUE */
	}struct_data;
	struct s_type_union {
		struct s_variable_bare *vars; /* QUEUE */
		union u_type_data *data;
	}union_data;
	struct s_type_enum {
		struct s_const *consts; /* QUEUE */
	}enum_data;
};

union u_type_data {
	union u_base_type_data bdata;
	union u_struct_type_data sdata;
};

struct s_type {
	char name[DEFAULT_NAME_LENGTH];
	uint32 size;
	uint32 pointer_count;
	uint32 type;
#define TYPE_INT         1
#define TYPE_UINT        2
#define TYPE_CHAR        3
#define TYPE_UCHAR       4
#define TYPE_SHORT       5
#define TYPE_USHORT      6
#define TYPE_LONG        7
#define TYPE_ULONG       8
#define TYPE_FLOAT       9
#define TYPE_DOUBLE     10
#define TYPE_LDOUBLE    11
#define TYPE_ADDRESS    12

#define TYPE_FUNCTION   102

#define TYPE_STRUCTURE  203
#define TYPE_UNION      204
#define TYPE_ENUM       205

#define TYPE_DEFINE     306
	union u_type_define {
		union u_struct_type_data sdef;
		struct s_type *udef;
		struct s_type_function {
			struct s_type *return_type;
			struct s_variable_bare *args; /* QUEUE */
		}fun;
	}u;
};

struct s_variable_bare {
	char name[DEFAULT_NAME_LENGTH];
	struct s_type *type;
};

struct s_symbol {
	char name[DEFAULT_NAME_LENGTH];
	uint32 type;
#define SYMBOL_TYPE_VARIABLE  1
#define SYMBOL_TYPE_CONST     2
#define SYMBOL_TYPE_MACRO     3
#define SYMBOL_TYPE_KEYWORD   4
};

struct s_keyword {
	char name[DEFAULT_NAME_LENGTH];
	uint32 type;
};

struct s_variable {
	char name[DEFAULT_NAME_LENGTH];
	uint32 type;
	struct s_type *vtype;
	union u_type_data data;
};

struct s_const {
	char name[DEFAULT_NAME_LENGTH];
	uint32 type;
	uint32 vtype;
	union u_type_data data;
};

struct s_macro {
	char name[DEFAULT_NAME_LENGTH];
	uint32 type;
	int from, end;
};

////////////////////////////////////////////////////////////////////////////


#define DEVICE_CLASS_BUTTON     5

struct CLASS;
struct OBJECT;
struct METHOD;
struct DEVICE;

typedef struct CLASS {
	struct METHOD *method;
	DWORD class_id;
}CLASS;

typedef struct OBJECT {
	struct METHOD *method;

	struct DEVICE *device; /* which device the object belong to */
	INT text_from, text_end; /* object map of the text resource */

	DWORD object_id;
	INT nX, nY;
	void *attribute;
}OBJECT;

typedef struct METHOD {
	int (*OnInitialize)(OBJECT *obj);
	int (*OnFinish)(OBJECT *obj);
	int (*OnTrace)(OBJECT *obj, struct DEVICE *dev, INT x, INT y, DWORD key);
	/* return 0 for success */
	int (*OnKeyPress)(OBJECT *obj, struct DEVICE *dev, INT x, INT y, DWORD key);
	int (*OnKeyRelease)(OBJECT *obj, struct DEVICE *dev, INT x, INT y, DWORD key);
	/* return 0 for success */
	int (*OnInput)(OBJECT *obj, struct DEVICE *dev, DWORD ch);
	int (*OnPaint)(OBJECT *obj, struct DEVICE *dev, SIZE *size);
	int (*OnUpdate)(OBJECT *obj, struct DEVICE *dev);
	int (*GetSize)(OBJECT *obj, SIZE *size);
}METHOD;

typedef struct DEVICE {
	/* hardware status */
	HWND hWnd;
	HDC hDC;
	BOOL bStretchMode;

	/* device global status */
	INT nOrigX;
	INT nOrigY;
	INT nWidth;
	INT nHeight;

	struct DEVICE_DRAW_STATUS {
		/*
		** screen attribute
		*/
//		INT nOrigX; /* no use */
//		INT nOrigY; /* no use */
		INT nWidth;
		INT nHeight;

		/*
		** screen consist of lines in row direction
		*/
//		INT nLineX; /* no use */
		INT nLineY;
		INT nLineWidth;
		INT nLineHeight;

		/*
		** line consist of objects in column direction
		*/
		INT nObjectX;
//		INT nObjectY; /* no use */
		INT nObjectWidth;
		INT nObjectHeight;

		/*
		** text object consist of text lines in row direction
		*/
		INT nTextLineX; /* if text line x and y equal to object x and y
						** then not text is inputed
						*/
		INT nTextLineY;
		INT nTextLineWidth;
		INT nTextLineHeight;

		/* object id initial from 0 */
		INT nID;
		INT text_offset; /* initial and used by update, no other usage */
	}content;
	struct DEVICE_ATTRIBUTE {
		INT nPenType;
		INT nPenWidth;
		COLORREF nPenColor;
		INT nBrushType;
		union DEVICE_BRUSH_ATTRIBUTE {
			TCHAR szFile[MAX_PATH]; /* bitmap file */
			DWORD dwId; /* bitmap resource id */
			INT nType; /* hatched brush */
			COLORREF color; /* solid color */
		}uBrushAttribute;
		TCHAR szTextFontName[LF_FACESIZE];
		INT nTable;
		INT nTextSize;
		BOOL bTextItalic;
		BOOL bTextBold;
		COLORREF nTextColor;
		COLORREF nBkColor;
		BOOL bBkNull;
	}attribute;
	struct CLASS *obj_class; /* ACTIVE CLASS QUEUE */
	struct OBJECT *objects; /* ACTIVE OBJECT QUEUE */
	char *text; /* STRING QUEUE */
	struct s_symbol *syms; /* QUEUE */
}DEVICE;

KAILIN_API
 int DeviceCreate(DEVICE *dev, HWND hWnd, INT nWidth, INT nHeight);
KAILIN_API
 int DeviceDestroy(DEVICE *dev);
KAILIN_API int DeviceAppendText(DEVICE *dev, const char *text, ...);
KAILIN_API int DeviceUpdate(DEVICE *dev);
KAILIN_API
 int DeviceOnPaint(DEVICE *dev, HWND hWnd, WPARAM wParam, LPARAM lParam);
KAILIN_API
 int DeviceOnSize(DEVICE *dev, HWND hWnd, WPARAM wParam, LPARAM lParam);
KAILIN_API
 int DeviceOnTrace(DEVICE *dev, INT x, INT y, DWORD key);
KAILIN_API
 int DeviceOnKeyPress(DEVICE *dev, INT x, INT y, DWORD key);
KAILIN_API
 int DeviceOnKeyRelease(DEVICE *dev, INT x, INT y, DWORD key);
KAILIN_API
 int DeviceOnInput(DEVICE *dev, DWORD ch);
KAILIN_API
 int DeviceSendUpdateMessage(DEVICE *dev);
KAILIN_API
 int DeviceSendPaintMessage(DEVICE *dev);
KAILIN_API
 int DeviceSendAppendTextMessage(DEVICE *dev, const char *text);
KAILIN_API
 int DeviceWriteString(DEVICE *dev, const char *fmt, ...);
KAILIN_API
 int DeviceDefWindowProc(DEVICE *dev, UINT uMsg, WPARAM wParam, LPARAM lParam, LRESULT *lResult);

#ifdef __cplusplus
};
#endif

#endif
