#ifndef __KEYS_H__
#define __KEYS_H__

#include "x/api.h"

struct s_value {
	struct s_value *previous, *next;
	int value_len;
	unsigned char value[1]; /* add one bytes for extra terminator character */
};

struct s_key {
	struct s_key *previous, *next;

	struct s_value *values;

	int type; /* key digital type */

	int name_len; /* key ascii name */
	unsigned char name[1]; /* add one bytes for extra terminator character */
};

/* GENERAL methods */
/*******************/

/* clean all the key(s) */
KAILIN_API int keys_clean(struct s_key **list);
/* count of key values */
KAILIN_API int keys_count_value(struct s_key *list);
/* count of keys */
KAILIN_API int keys_count(struct s_key *list);
/* test key value empty */
KAILIN_API int keys_empty_value(struct s_key *list);
/* test key list empty */
KAILIN_API int keys_empty(struct s_key *list);
/* wall through all the key(s), from first to end */
KAILIN_API int keys_walk(struct s_key *list, struct s_key **node);
/* wall through all the value(s), from first to end */
KAILIN_API int keys_walk_value(struct s_key *node, struct s_value **value);
/* get key newest value */
KAILIN_API int keys_get_value(struct s_key *key, struct s_value **value);
/* get key oldest value */
KAILIN_API int keys_get_value_0(struct s_key *key, struct s_value **value);
/* set value */
KAILIN_API int keys_set_value(struct s_key *key, const void *value, int value_len);
/* create a empty value */
KAILIN_API int keys_create_value(struct s_value **value, int value_len);
/* free value */
KAILIN_API int keys_destroy_value(struct s_value **value);
KAILIN_API int keys_destroy(struct s_key **key);
/* put value to key */
KAILIN_API int keys_put_value(struct s_key *key, struct s_value *value);
/* dup keys to environ serial */
KAILIN_API int keys_make_env(struct s_key *list, char **env, int *olen);
/* copy source keys to target */
KAILIN_API int keys_copy(struct s_key **tar, const struct s_key *src);
/* set one key to the list */
KAILIN_API int keys_set(struct s_key **list, struct s_key *key);

/* TYPE methods */
/****************/

/* test wether a key exist */
KAILIN_API int keys_exist(struct s_key *list, int type);
/* find the specified key */
KAILIN_API int keys_get(struct s_key *list, int type, struct s_key **node);
/* create a new key */
KAILIN_API int keys_create(struct s_key **key, int type);
/* create a new key or value */
KAILIN_API int keys_new(struct s_key **list, int type, const void *value, int value_len);
/* delete the spcified key */
KAILIN_API int keys_delete(struct s_key **list, int type);

/* NAME methods */
/****************/

/* test wether a key exist */
KAILIN_API int keys_exist_n(struct s_key *list, const char *name, int len);
/* find the specified key */
KAILIN_API int keys_get_n(struct s_key *list, const char *name, int len, struct s_key **node);
/* create a new key */
KAILIN_API int keys_create_n(struct s_key **key, const char *name, int len);
/* create a new key or value */
KAILIN_API int keys_new_n(struct s_key **list, const char *name, int len, const void *value, int value_len);
/* create a empty key */
KAILIN_API int keys_create_empty_n(struct s_key **key, int name_len);
/* delete the spcified key */
KAILIN_API int keys_delete_n(struct s_key **list, const char *name, int len);
/* set key type from name table after creation of key list */
KAILIN_API int keys_set_type_n(struct s_key *list, const char *names[], int names_len);

#endif
