#ifndef __SYS_H__
#define __SYS_H__

#include "x/api.h"

/* ansi c lib */

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* win32 api */

#include <winsock2.h> /* winsock 2, must link ws2_32.lib */
#include <ws2tcpip.h> /* add ip v6 support */
#include <mswsock.h>  /* add TransmitFile, AcceptEx, must link mswsock.lib */
#include <windows.h>
#include <mmsystem.h>

/* configure */

#define FILE_FETCH_BUFFER_SIZE     32

/* MACRO */

#define ZERO(p) (memset(&(p),0,sizeof(p)))

/* constant */

#define FILE_OPEN_WRITABLE     1

/* data type */

struct s_time {
	int year;
	int month;
	int day;
	int week_day;
	int hour;
	int minute;
	int second;
	int milli_second;
};

union u_sockaddr {
	struct sockaddr addr;
	struct sockaddr_in addr_in;
	struct sockaddr_in6 addr_in6;
};

struct s_file;

struct s_file_operation {
	OVERLAPPED overlapped;
	struct s_file *file;
};

struct s_iocp {
	HANDLE hCompletion;
};

struct s_file {
//	OVERLAPPED overlapped;
	HANDLE hFile;
	DWORD dwWaitMilliSeconds;
	/* fetch buffer is a round buffer */
	unsigned char fetch[FILE_FETCH_BUFFER_SIZE];
	int fetch_len, fetch_off;
	/* IOCP special fields */
	struct s_file_operation file_op[4];
#define IOCP_OP_READ   0
#define IOCP_OP_WRITE  1
#define IOCP_OP_ACCEPT 2
#define IOCP_OP_POST   3
#define IOCP_OP_CTRL   4
	struct s_iocp *completion;
};

struct s_pcb {
	HANDLE hProcess;
	HANDLE hThread;
	DWORD dwWaitTime;
	struct s_file pipe_stdout;
	struct s_file pipe_stdin;
	struct s_file pipe_stderr;
};

struct s_file_stat {
	unsigned long file_size;
	unsigned long file_flags;
#define FILE_FLAGS_DIRECTORY            1
	struct s_time modify_time;
};

struct s_thread {
	HANDLE hThread;
};

struct s_timer {
	struct s_timer *previous, *next;
	unsigned timer;
	unsigned time_interval;
	int (*func)(void *p);
	void *param;
};

struct s_timer_env {
	struct s_timer *queue;
	int resolution;
	unsigned timer_id;
	unsigned period;
	CRITICAL_SECTION lock;
};

struct s_console {
	HANDLE hMutex;
};

/* thread function */

KAILIN_API int sys_thread_create(struct s_thread *thread, int (*func)(void *p), void *param);
KAILIN_API int sys_thread_wait(struct s_thread *thread, int *exitcode, int ms);

/* io complete port api */

KAILIN_API int sys_iocp_open(struct s_iocp *iocp);
KAILIN_API int sys_iocp_close(struct s_iocp *iocp);
KAILIN_API int sys_iocp_bind(struct s_iocp *iocp, struct s_file *file);
KAILIN_API int sys_iocp_ctrl(struct s_iocp *iocp, int type);
KAILIN_API int sys_iocp_get(struct s_iocp *iocp, struct s_file **father, struct s_file **file, long *bytes);
KAILIN_API int sys_iocp_post(struct s_file *file, int type);
KAILIN_API int sys_iocp_read(struct s_file *file, char *buff, long len);
KAILIN_API int sys_iocp_receive(struct s_file *file, char *buff, long len);
KAILIN_API int sys_iocp_write(struct s_file *file, const char *buff, long len);
KAILIN_API int sys_iocp_send(struct s_file *file, const char *buff, long len);
KAILIN_API int sys_iocp_accept(struct s_file *file, struct s_file *out_file, char *buff, int buff_len);
KAILIN_API int sys_iocp_accept_address(char *buff, int buff_len, union u_sockaddr *local, union u_sockaddr *remote);

/* file system api */

KAILIN_API int sys_file_length(const char *path, long *len);
KAILIN_API int sys_file_test(const char *path, struct s_file_stat *stat);
KAILIN_API int sys_file_is_file(const char *path, struct s_file_stat *stat);
KAILIN_API int sys_file_is_dir(const char *path);
KAILIN_API int sys_file_is_exist(const char *path);
KAILIN_API int sys_file_duplicate(struct s_file *file, struct s_file *out_file);
KAILIN_API int sys_file_open(struct s_file *file, const char *path, int flags);
KAILIN_API int sys_file_close(struct s_file *file);
KAILIN_API int sys_file_set_timeout(struct s_file *file, unsigned long ms);
/* base io methods */
KAILIN_API int sys_file_read(struct s_file *file, void *buff, int buff_len, int *read_len);
KAILIN_API int sys_file_write(struct s_file *file, const void *buff, int buff_len, int *write_len);
KAILIN_API int sys_file_write_buff(struct s_file *file, struct s_vstr *buff);
/* fetch buffer methods */
KAILIN_API int sys_file_ungets(struct s_file *file, const char *s, int len);
KAILIN_API int sys_file_ungetc(struct s_file *file, int c);
KAILIN_API char *sys_file_get_fetch(struct s_file *file, char *buff, int buff_len);
/* character based methods */
KAILIN_API int sys_file_getc(struct s_file *file);
KAILIN_API int sys_file_putc(struct s_file *file, int c);
/* line based methods, NOTE: gets/puts will transfer line end character(s) to/from '\n' */
KAILIN_API int sys_file_puts(struct s_file *file, const char *s);
KAILIN_API char *sys_file_gets(struct s_file *file, char *buff, int buff_len);
/* terminator base methods */
KAILIN_API char *sys_file_get_t(struct s_file *file, char *buff, int buff_len, const char *term, char t);
KAILIN_API int sys_file_put_t(struct s_file *file, const char *s, const char *term, char t);
/* raw out methods */
KAILIN_API int sys_file_put_raw(struct s_file *file, const char *s);

/* general socket api */

KAILIN_API int sys_sock_sys_init(const char **msg);
KAILIN_API int sys_sock_sys_exit(void);
KAILIN_API int sys_sock_sys_get_hostname(char *buff, int limit);
KAILIN_API int sys_sock_sys_get_peername(union u_sockaddr *addr, char *buff, int limit);
KAILIN_API int sys_sock_sys_enum_nic(void *param, unsigned short port, int (*func)(union u_sockaddr *addr, void *param));
KAILIN_API int sys_sock_address(union u_sockaddr *addr, const char *ip, unsigned short port);
KAILIN_API int sys_sock_close(struct s_file *sock);
KAILIN_API int sys_sock_shutdown(struct s_file *sock);

/* tcp socket api */

KAILIN_API int sys_sock_listen(struct s_file *sock, union u_sockaddr *addr, int back_log);
KAILIN_API int sys_sock_accept(struct s_file *sock, union u_sockaddr *out_addr, struct s_file *out_sock);
KAILIN_API int sys_sock_connect(struct s_file *sock, union u_sockaddr *addr);
KAILIN_API int sys_sock_set_timeout(struct s_file *sock, int ms);

/* process api */

KAILIN_API int sys_proc_open(struct s_pcb *pcb, char *cmdline, const char *dir, char *env);
KAILIN_API int sys_proc_close(struct s_pcb *pcb);
KAILIN_API int sys_proc_wait(struct s_pcb *pcb);
KAILIN_API int sys_proc_set_time(struct s_pcb *pcb, unsigned long ms);
KAILIN_API struct s_file *sys_proc_stdin(struct s_pcb *pcb);
KAILIN_API struct s_file *sys_proc_stdout(struct s_pcb *pcb);
KAILIN_API struct s_file *sys_proc_stderr(struct s_pcb *pcb);

/* system infomation */

KAILIN_API int sys_info_processors();

/* timer api */

KAILIN_API int sys_timer_init(struct s_timer_env *env);
KAILIN_API int sys_timer_exit(struct s_timer_env *env);
KAILIN_API int sys_timer_kill(struct s_timer_env *env, struct s_timer **timer);
KAILIN_API int sys_timer_set(struct s_timer_env *env, struct s_timer **timer, int ms, void *param, int (*func)(void *p));

/* console output */

KAILIN_API int sys_con_init(struct s_console *con);
KAILIN_API int sys_con_exit(struct s_console *con);

KAILIN_API int sys_vprintf(struct s_console *con, const char *fmt, va_list vargs);
KAILIN_API int sys_printf(struct s_console *con, const char *fmt, ...);

/* memory */

KAILIN_API int sys_malloc(void **ptr, int size);
KAILIN_API int sys_mfree(void **ptr);

#endif
